////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  ____  ____  ____   __    ___  _____ 
// |  _ \|  _ \/ ___| / /_  / _ \|___ / 
// | | | | | | \___ \| '_ \| | | | |_ \ 
// | |_| | |_| |___) | (_) | |_| |___) |
// |____/|____/|____/ \___/ \___/|____/ 
//                                               
//  __  __           _       _      
// |  \/  | ___   __| |_   _| | ___ 
// | |\/| |/ _ \ / _` | | | | |/ _ \
// | |  | | (_) | (_| | |_| | |  __/
// |_|  |_|\___/ \__,_|\__,_|_|\___|
//                                  
//  FOR NOKIA 3310 LCD SCREEN
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//    
//  VERSION:
//    1.01
//
//  RELEASE DATE:
//    SAT 16th JAN 2010
//
//  CODED BY:
//    Fraser
//
//  URL:
//   http://www.overunityresearch.com/index.php?topic=12
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  COPYRIGHT:
//    Copyright (c) 2010 www.OverUnityResearch.com
//    All rights reserved.
//
//    Redistribution and use in source and binary forms, with or without modification,
//    are permitted provided that the following conditions are met:
//
//    [1] Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
//
//    [2] Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation and/
//    or other materials provided with the distribution.
//
//    [3] Commercial users must first seek a commercial license before product release.
//    this can be done by contacting fraser@overunityresearch.com.
//    Commercial users can develop under this existing license.
//
//    DISCLAIMER:
//      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
//      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//      THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//      ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
//      FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//      THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//      EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  ATMEL ATMEGA168 PINOUT:
//                                +----\/----+   
//       (PCINT14/RESET) PC6      |01      28| (A5) PC5 (ADC5/SCL/PCINT13) 
//         (PCINT16/RXD) PD0 (00) |02      27| (A4) PC4 (ADC4/SDA/PCINT12) 
//         (PCINT17/TXD) PD1 (01) |03      26| (A3) PC3 (ADC3/PCINT11) 
//        (PCINT18/INT0) PD2 (02) |04      25| (A2) PC2 (ADC2/PCINT10) 
//   (PCINT19/OC2B/INT1) PD3 (03) |05      24| (A1) PC1 (ADC1/PCINT9) 
//      (PCINT20/XCK/T0) PD4 (04) |06      23| (A0) PC0 (ADC0/PCINT8) 
//                       VCC      |07      22|      GND 
//                       GND      |08      21|      AREF 
//  (PCINT6/XTAL1/TOSC1) PB6      |09      20|      AVCC 
//  (PCINT7/XTAL2/TOSC2) PB7      |10      19| (13) PB5 (SCK/PCINT5) 
//     (PCINT21/OC0B/T1) PD5 (05) |11      18| (12) PB4 (MISO/PCINT4) 
//   (PCINT22/OC0A/AIN0) PD6 (06) |12      17| (11) PB3 (MOSI/OC2A/PCINT3) 
//        (PCINT23/AIN1) PD7 (07) |13      16| (10) PB2 (SS/OC1B/PCINT2) 
//    (PCINT0/CLKO/ICP1) PB0 (08) |14      15| (09) PB1 (OC1A/PCINT1)
//                                +----------+
//
//  NOTE: Bracketed numbers (xx) represent Arduino/Freeduino board IO pinouts.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//ARDUINO BOARD IO PINS:
//SERIAL RX: 0
//SERIAL TX: 1
const int PIN_JOG_GREY_A = 2;
const int PIN_JOG_GREY_B = 3;
const int PIN_JOG_BUTTON = 4;
const int PIN_DDS_CLK1 = 5;
const int PIN_DDS_CLK2 = 6;
const int PIN_DDS_CLK3 = 7;
//USED BY NOKIA SCREEN: 8
//USED BY NOKIA SCREEN: 9
//USED BY NOKIA SCREEN: 10
//USED BY NOKIA SCREEN: 11
const int PIN_BUZZER = 12;
//USED BY NOKIA SCREEN: 13
const int PIN_ADC_JOYPAD = 0;//[analog pin 0] (used to read joystick on nokia screen shield)
const int PIN_ADC_UNUSED = 1;//[analog pin 1]
const int PIN_ADC_PSU = 2;   //[analog pin 2] (used to detect system powerdown)
const int PIN_DDS_DATA = 17; //[analog pin 3] (used as a digital output pin)
const int PIN_DDS_LOAD = 18; //[analog pin 4] (used as a digital output pin)
const int PIN_DDS_TRIG = 19; //[analog pin 5] (used as a digital output pin)

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Includes
#include <avr/pgmspace.h>
#include <interrupt.h>
#include <io.h>
#include <EEPROM.h>// for persistent system state
#include <LCD_Nokia_3310.h>// this file must be put in your arduino library folder

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Hack to fix core API compile errors (which shouldn't have been defined over base defines)
#undef int
#undef abs
#undef double
#undef float
#undef round

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Macros
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif

#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#define ENABLE_JOG_INTERRUPT (EIMSK |= 1);(EIMSK |= 2)
#define DISABLE_JOG_INTERRUPT (EIMSK &= 2);(EIMSK &= 1)

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SYSTEM Constants:
const long DDS_CLOCK = 180000000;//180MHz (after DDS 6x clock multiplier on the 30MHz crystal clock source)
const int NUM_CHANNELS = 3;
const long MIN_FREQUENCY = 0;//0Hz
const long MAX_FREQUENCY = 70000000;//70MHz
const int MAIN_LOOP_DELAY = 75;//millisecinds

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GUI RELATED:
const int GUI_SCREEN_MENU = 0;
const int GUI_SCREEN_FREQUENCY = 1;
const int GUI_SCREEN_PHASE = 2;
const int GUI_SCREEN_SWEEP = 3;
const int GUI_SCREEN_HARMONIC = 4;
const int GUI_SCREEN_MEMORY = 5;
const int NUM_GUI_SCREENS = 6;

const int CURSOR_WIDTH = 3;
const int CURSOR_HEIGHT = 7;

#define BEEP_MOVE       Beep(500,20)
#define BEEP_CURSOR_LO  Beep(1000,10) 
#define BEEP_CURSOR_HI  Beep(1500,10)
#define BEEP_DEC        Beep(2000,1)
#define BEEP_INC        Beep(4000,1)
#define BEEP_MENU       Beep(500,10);Beep(1000,10)
#define BEEP_ERROR      Beep(100,200)

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Objects
LCD_Nokia_3310 LCD;

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Variables
struct
{
  bool bSweeping;
  bool bCursorState;
  bool bCursorStateOld;
  int  iCursorPos;
  int  iCursorPosOld;
  int  iGuiScreen;
  int  iCursorMax[NUM_GUI_SCREENS];
  unsigned long ulLastJog;
}
t;//temporary working vars

struct
{
  long lFreq[NUM_CHANNELS];
  long lSweepFreq[3][NUM_CHANNELS];
  bool bPiezoEnabled;
}
v;//vars to save to EEPROM

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialise
void setup()
{
  //Timer 2 setup:
  TCCR2A = 0;
  TCCR2B = 1<<CS22 | 1<<CS21;
  //Timer2 Overflow Interrupt Enable  
  TIMSK2 = 1<<TOIE2;
  //reset timer
  TCNT2 = 0;

  //jogbutton:
  pinMode(PIN_JOG_BUTTON, INPUT);
  digitalWrite(PIN_JOG_BUTTON, LOW);//pulldown

  //jogwheel:
  pinMode(PIN_JOG_GREY_A, INPUT);
  digitalWrite(PIN_JOG_GREY_A, HIGH);//pullup
  EICRA |= (1 << ISC01);// Signal fall triggers INT0
  (EIMSK |= 1);// Global Enable INT0

  pinMode(PIN_JOG_GREY_B, INPUT);
  digitalWrite(PIN_JOG_GREY_B, HIGH);//pullup
  EICRA |= (1 << ISC11);// Signal fall triggers INT1
  (EIMSK |= 2);// Global Enable INT1

  //buzzer:
  v.bPiezoEnabled = true;

  //DDS control:
  pinMode (PIN_DDS_DATA, OUTPUT);
  pinMode (PIN_DDS_LOAD, OUTPUT);
  pinMode (PIN_DDS_CLK1, OUTPUT);
  pinMode (PIN_DDS_CLK2, OUTPUT);
  pinMode (PIN_DDS_CLK3, OUTPUT);

  LCD.Initialise();
  GuiScreenSplash();

  //persistent var setup:
  for(int r = 0; r < 10; r++)
  {
    for(int i = 0; i < NUM_CHANNELS; i++)
    {
      v.lSweepFreq[2][i] = 1;
      v.lFreq[i] = 10000000;
      DDS_SetFrequency(i, v.lFreq[i]);
    }
    delay(1);
  }
  //temp vars setup:
  t.ulLastJog = 0;
  t.bSweeping = false;
  t.bCursorState = false;
  t.iCursorPos = 7;
  t.iGuiScreen = GUI_SCREEN_FREQUENCY;
  t.iCursorPosOld = -1;
  t.bCursorStateOld = false;
  t.iCursorMax[GUI_SCREEN_MENU] = 4;
  t.iCursorMax[GUI_SCREEN_FREQUENCY] = 24;
  t.iCursorMax[GUI_SCREEN_PHASE] = 3;
  t.iCursorMax[GUI_SCREEN_SWEEP] = 3;
  t.iCursorMax[GUI_SCREEN_HARMONIC] = 3;
  t.iCursorMax[GUI_SCREEN_MEMORY] = 3;

  GuiScreenFrequency(true);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   __  __       _         _                      
//  |  \/  | __ _(_)_ __   | |    ___   ___  _ __  
//  | |\/| |/ _` | | '_ \  | |   / _ \ / _ \| '_ \ 
//  | |  | | (_| | | | | | | |__| (_) | (_) | |_) |
//  |_|  |_|\__,_|_|_| |_| |_____\___/ \___/| .__/ 
//                                          |_|    
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
void loop()
{
  if(t.iGuiScreen == GUI_SCREEN_MENU)
    GuiScreenMenu(false);
  else if(t.iGuiScreen == GUI_SCREEN_FREQUENCY)
    GuiScreenFrequency(false);
  else if(t.iGuiScreen == GUI_SCREEN_PHASE)
    GuiScreenPhase(false);
  else if(t.iGuiScreen == GUI_SCREEN_SWEEP)
  {
    GuiScreenSweep(false);
    if(t.bSweeping)
      PerformSweep();
  }
  else if(t.iGuiScreen == GUI_SCREEN_HARMONIC)
    GuiScreenHarmonic(false);
  else if(t.iGuiScreen == GUI_SCREEN_MEMORY)
    GuiScreenMemory(false);

  PollJoyPad();

  LCD.RenderBuffer();

  delay(MAIN_LOOP_DELAY);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//    ____ _   _ ___   ____                     _             
//   / ___| | | |_ _| |  _ \ _ __ __ ___      _(_)_ __   __ _ 
//  | |  _| | | || |  | | | | '__/ _` \ \ /\ / / | '_ \ / _` |
//  | |_| | |_| || |  | |_| | | | (_| |\ V  V /| | | | | (_| |
//   \____|\___/|___| |____/|_|  \__,_| \_/\_/ |_|_| |_|\__, |
//                                                      |___/ 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Show loading splash screen
void GuiScreenSplash(void)
{
  LCD.Triangle(42,0,26,16,58,16,false);
  LCD.Triangle(26,16,10,32,42,32,false);
  LCD.Triangle(58,16,42,32,74,32,false);
  LCD.SmallString(18, 35, "TRIFORCE", -1);
  LCD.Rectangle(10,43,74,47,false);
  for(int x=10; x <= 74; x++)
  {  
    LCD.Line(x,44,x,47,false);
    delay(10);
    LCD.RenderBuffer();
    Beep(x*10 + 1000,5);
  }
  delay(200);
  LCD.ClearBuffer();
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Common init stuff when entering a screen
void InitialiseScreen(int iGuiScreen)
{
  t.iGuiScreen = iGuiScreen;  
  t.iCursorPos = 0;
  t.iCursorPosOld = -1;
  t.bCursorState = false;
  t.bCursorStateOld = true;
  LCD.ClearBuffer();
  BEEP_MENU;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the GUI screen as required
void GuiScreenMenu(bool bForceInit)
{
  const int LEFT_POS = 6;
  const int TOP_POS = 0;
  const int INTER_LINE = 8;

  //Screen initialisation:
  if(bForceInit)
  {
    InitialiseScreen(GUI_SCREEN_MENU);
    //draw menu items:
    int iY = TOP_POS;
    LCD.SmallString(LEFT_POS,iY,"    MENU    ", 100);
    iY += INTER_LINE;
    LCD.SmallString(LEFT_POS,iY,"ADJUST FREQS", -1);
    iY += INTER_LINE;
    LCD.SmallString(LEFT_POS,iY,"ADJUST PHASE", -1);
    iY += INTER_LINE;
    LCD.SmallString(LEFT_POS,iY,"SWEEP FREQ 1", -1);
    iY += INTER_LINE;
    LCD.SmallString(LEFT_POS,iY,"HARMONIC LOC", -1);
    iY += INTER_LINE;
    LCD.SmallString(LEFT_POS,iY,"SETUP MEMORY", -1);

    t.iCursorPos = 4;//start at "1D1" position
  }
  //Cursor:
  if(t.iCursorPosOld != t.iCursorPos)
  { 
    DISABLE_JOG_INTERRUPT;
    const int X1 = 0;
    const int X2 = 80;
    const int Y = 40;
    int y;
    //delete old
    y = Y - (t.iCursorPosOld * INTER_LINE);
    LCD.Box(X1,y,X1+CURSOR_WIDTH,y+CURSOR_HEIGHT,true);
    LCD.Box(X2,y,X2+CURSOR_WIDTH,y+CURSOR_HEIGHT,true);
    //draw new
    y = Y - (t.iCursorPos * INTER_LINE);
    LCD.Box(X1,y,X1+CURSOR_WIDTH,y+CURSOR_HEIGHT,false);
    LCD.Box(X2,y,X2+CURSOR_WIDTH,y+CURSOR_HEIGHT,false);
    //remember this pos
    t.iCursorPosOld = t.iCursorPos;
    ENABLE_JOG_INTERRUPT;
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the GUI screen as required
static unsigned char PADLOCK_BITMAP[] = {
  0x3C, 0x42, 0x42, 0xFF, 0xFF, 0xE7, 0xE7, 0xFF};
void GuiScreenFrequency(bool bForceInit)
{
  const unsigned char LEFT_POS = 0;
  const unsigned char TOP_POS = 0;
  const unsigned char INTER_LINE = 13;
  static long lFreqOld[NUM_CHANNELS];  
  //////////////////////////
  //Screen initialisation:
  if(bForceInit)
  {
    InitialiseScreen(GUI_SCREEN_FREQUENCY);
    //draw menu
    LCD.Bitmap(74, 39, 8, 8, PADLOCK_BITMAP);
    LCD.SmallString(30, 40, "MENU", -1);
    //force 3 channel freq redraw
    for(int i = 0; i < NUM_CHANNELS; i++)
      lFreqOld[i] = -1;  
  }

  ///////////////////////////////////////
  //Freq display update as required:
  for(int i = 0; i < NUM_CHANNELS; i++)
  {  //Look for a changed freq:
    if(lFreqOld[i] != v.lFreq[i] || t.iCursorPosOld != t.iCursorPos)
    { //Number has changed, redraw:
      lFreqOld[i] = v.lFreq[i];
      int y = TOP_POS + i * INTER_LINE;

      int iInvert = -1;
      if(t.iCursorPos > 0)
        if((t.iCursorPos-1) / 8 == i)
          iInvert = (t.iCursorPos-1) % 8;

      LCD.LargeNumber(LEFT_POS, y, v.lFreq[i], iInvert);
    }
  }
  ///////////////////
  //Cursor:
  const int MX1 = 25;
  const int MX2 = 56;
  const int MY = 40;
  if(t.iCursorPosOld != t.iCursorPos)
  { 
    DISABLE_JOG_INTERRUPT;
    //delete old
    if(t.iCursorPosOld == 0)
    {
      LCD.Box(MX1,MY,MX1+CURSOR_WIDTH,MY+CURSOR_HEIGHT,true);
      LCD.Box(MX2,MY,MX2+CURSOR_WIDTH,MY+CURSOR_HEIGHT,true);
    }
    //draw new
    if(t.iCursorPos == 0)
    {
      LCD.Box(MX1,MY,MX1+CURSOR_WIDTH,MY+CURSOR_HEIGHT,false);
      LCD.Box(MX2,MY,MX2+CURSOR_WIDTH,MY+CURSOR_HEIGHT,false);
    }
    t.iCursorPosOld = t.iCursorPos;
    ENABLE_JOG_INTERRUPT;
  }
  if(t.bCursorStateOld != t.bCursorState)
  {
    DISABLE_JOG_INTERRUPT;
    LCD.Box(2,40,10,43,!t.bCursorState);  
    LCD.Box(2,45,10,48,t.bCursorState); 
    t.bCursorStateOld = t.bCursorState;
    ENABLE_JOG_INTERRUPT;
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the GUI screen as required
void GuiScreenPhase(bool bForceInit)
{ 
  const int LEFT_POS = 6;
  const int TOP_POS = 0;
  const int INTER_LINE = 8;

  //Screen initialisation:
  if(bForceInit)
  {
    InitialiseScreen(GUI_SCREEN_PHASE);
    LCD.SmallString(0,0,"PHASE:",-1);
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the GUI screen as required
void GuiScreenSweep(bool bForceInit)
{ 
  const int LEFT_POS = 6;
  const int TOP_POS = 0;
  const int INTER_LINE = 8;

  //Screen initialisation:
  if(bForceInit)
  {
    InitialiseScreen(GUI_SCREEN_SWEEP);
    LCD.SmallString(0,0,"SWEEP:",-1);
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the GUI screen as required
void GuiScreenHarmonic(bool bForceInit)
{ 
  const int LEFT_POS = 6;
  const int TOP_POS = 0;
  const int INTER_LINE = 8;

  //Screen initialisation:
  if(bForceInit)
  {
    InitialiseScreen(GUI_SCREEN_HARMONIC);
    LCD.SmallString(0,0,"HARMONIC:",-1);
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the GUI screen as required
void GuiScreenMemory(bool bForceInit)
{ 
  const int LEFT_POS = 6;
  const int TOP_POS = 0;
  const int INTER_LINE = 8;

  //Screen initialisation:
  if(bForceInit)
  {
    InitialiseScreen(GUI_SCREEN_MEMORY);
    LCD.SmallString(0, 0,"MEMORY:",-1);
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Redraw the menu as required
// THIS CODE UNUSED AT THE MOMENT
/*
const int BOTTOM_Y = 119;
 void GuiMenuSweep(bool bForceInit)
 {
 const int MENU_BAR = 24;
 const int LEFT_POS = 0;
 const int TITLE_Y = 10;
 const int TOP_DIGITS = 28;
 const int INTER_LINE = 10;
 
 static long lSweepFreqStart[NUM_CHANNELS];
 static long lSweepFreqEnd[NUM_CHANNELS];
 static long lSweepFreqRes[NUM_CHANNELS];
 
 //////////////////////////
 //Screen initialisation:
 int iDimensions;
 if(bForceInit)
 {
 InitialiseMenu(t.iMenuType);
 iDimensions = t.iMenuType - MENU_1D_SWEEP + 1;  
 
 LCD.SmallString("ROOT", LEFT_POS + 16, BOTTOM_Y, COLOR_TEXT, COLOR_PAPER);
 LCD.PrintString("RUN", 90, BOTTOM_Y, COLOR_TEXT, COLOR_PAPER);
 
 //force 3 channel redraw
 for(int i = 0; i < NUM_CHANNELS; i++)
 {
 lSweepFreqStart[i] = -1;
 lSweepFreqEnd[i] = -1;
 lSweepFreqRes[i] = -1;
 }
 t.iCursorPos = 1;//start at "run" position
 }
 
 ///////////////////////////////////////
 //Freq display update as required:
 int iY = TOP_DIGITS;
 const int PREFIX_LEFT = 4;
 const int DIGIT_LEFT = 64;
 for(int i = 0; i < iDimensions; i++)
 { 
 //Look for a changed value:
 if(lSweepFreqStart[i] != v.lSweepFreq[0][i])
 { 
 LCD.PrintString("STA", PREFIX_LEFT, iY, COLOR_TEXT, COLOR_PAPER);
 LCD.PrintNumber(i+1, PREFIX_LEFT + 32, iY, COLOR_TEXT, COLOR_PAPER,1);
 LCD.PrintNumber(v.lSweepFreq[0][i], DIGIT_LEFT, iY, COLOR_TEXT, COLOR_PAPER, 8);
 lSweepFreqStart[i] = v.lSweepFreq[0][i];
 }
 iY += INTER_LINE;
 if(lSweepFreqEnd[i] != v.lSweepFreq[1][i])
 { 
 LCD.PrintString("END", PREFIX_LEFT, iY, COLOR_TEXT, COLOR_PAPER);
 LCD.PrintNumber(i+1, PREFIX_LEFT + 32, iY, COLOR_TEXT, COLOR_PAPER,1);
 LCD.PrintNumber(v.lSweepFreq[1][i], DIGIT_LEFT, iY, COLOR_TEXT, COLOR_PAPER, 8);
 lSweepFreqEnd[i] = v.lSweepFreq[1][i];
 }
 iY += INTER_LINE;
 if(iDimensions >= 3)
 {
 if(lSweepFreqRes[i] != v.lSweepFreq[2][i])
 { 
 LCD.PrintString("RES", PREFIX_LEFT, iY, COLOR_TEXT, COLOR_PAPER);
 LCD.PrintNumber(i+1, PREFIX_LEFT + 32, iY, COLOR_TEXT, COLOR_PAPER,1);
 LCD.PrintNumber(v.lSweepFreq[2][i], DIGIT_LEFT, iY, COLOR_TEXT, COLOR_PAPER, 8);
 lSweepFreqRes[i] = v.lSweepFreq[2][i];
 }
 iY += INTER_LINE;
 }  
 }
 ///////////////////
 //Cursor:
 const int MX1 = 7;
 const int MX2 = 52;
 const int MX3 = 77;
 const int MX4 = 115;
 const int MY=119;
 if(t.iCursorPosOld != t.iCursorPos || t.bCursorStateOld != t.bCursorState)
 { 
 DISABLE_JOG_INTERRUPT;
 //delete old
 if(t.iCursorPosOld == 0)
 {
 LCD.Box(MX1,MY,MX1+CURSOR_WIDTH,MY+CURSOR_HEIGHT,COLOR_PAPER);
 LCD.Box(MX2,MY,MX2+CURSOR_WIDTH,MY+CURSOR_HEIGHT,COLOR_PAPER);
 }
 if(t.iCursorPosOld == 1)
 {
 LCD.Box(MX3,MY,MX3+CURSOR_WIDTH,MY+CURSOR_HEIGHT,COLOR_PAPER);
 LCD.Box(MX4,MY,MX4+CURSOR_WIDTH,MY+CURSOR_HEIGHT,COLOR_PAPER);
 }
 else if(t.iCursorPosOld > 1)
 {
 int x1 = ((t.iCursorPosOld-2) % 8) * 8 + DIGIT_LEFT;
 int x2 = x1 + 6;
 int y1 = ((t.iCursorPosOld-2) / 8) * INTER_LINE + TOP_DIGITS - 3;
 int y2 = y1 + 2;
 LCD.Box(x1,y1,x2,y2,COLOR_PAPER);
 LCD.Box(x1,y1+INTER_LINE,x2,y2+INTER_LINE,COLOR_PAPER);
 }
 
 //draw new
 int col = COLOR_GFX1;
 if(t.bCursorState)
 col = COLOR_GFX2;
 
 if(t.iCursorPos == 0)
 {
 LCD.Box(MX1,MY,MX1+CURSOR_WIDTH,MY+CURSOR_HEIGHT,col);
 LCD.Box(MX2,MY,MX2+CURSOR_WIDTH,MY+CURSOR_HEIGHT,col);
 }
 if(t.iCursorPos == 1)
 {
 LCD.Box(MX3,MY,MX3+CURSOR_WIDTH,MY+CURSOR_HEIGHT,col);
 LCD.Box(MX4,MY,MX4+CURSOR_WIDTH,MY+CURSOR_HEIGHT,col);
 }
 else if(t.iCursorPos > 1)
 {
 int x1 = ((t.iCursorPos-2) % 8) * 8 + DIGIT_LEFT;
 int x2 = x1 + 6;
 int y1 = ((t.iCursorPos-2) / 8) * INTER_LINE + TOP_DIGITS - 3;
 int y2 = y1 + 2;
 LCD.Box(x1,y1,x2,y2,col);
 LCD.Box(x1,y1+INTER_LINE,x2,y2+INTER_LINE,col);
 }
 t.iCursorPosOld = t.iCursorPos;
 t.bCursorStateOld = t.bCursorState;
 ENABLE_JOG_INTERRUPT;
 }
 }//*/
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//    ____ _   _ ___   _   _                 _ _ _             
//   / ___| | | |_ _| | | | | __ _ _ __   __| | (_)_ __   __ _ 
//  | |  _| | | || |  | |_| |/ _` | '_ \ / _` | | | '_ \ / _` |
//  | |_| | |_| || |  |  _  | (_| | | | | (_| | | | | | | (_| |
//   \____|\___/|___| |_| |_|\__,_|_| |_|\__,_|_|_|_| |_|\__, |
//                                                       |___/ 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// React to a user press in a GUI screen
void GuiScreenPressed(void)
{
  ///////////////////////////////////
  // SETUP MENU:  
  if(t.iGuiScreen == GUI_SCREEN_MENU)
  {
    if(t.iCursorPos == 4)
      GuiScreenFrequency(true);
    else if(t.iCursorPos == 3)
      GuiScreenPhase(true);
    else if(t.iCursorPos == 2)
      GuiScreenSweep(true);
    else if(t.iCursorPos == 1)
      GuiScreenHarmonic(true);
    else if(t.iCursorPos == 0)
      GuiScreenMemory(true);
  }
  ///////////////////////////////////
  // FREQUENCY:  
  else if(t.iGuiScreen == GUI_SCREEN_FREQUENCY)
  {
    if(t.iCursorPos == 0)
      GuiScreenMenu(true);
    else
      ToggleCursor();
  }
  ///////////////////////////////////
  // PHASE:
  else if(t.iGuiScreen == GUI_SCREEN_PHASE)
  {
    GuiScreenMenu(true);
  }
  ///////////////////////////////////
  // SWEEP:
  else if(t.iGuiScreen == GUI_SCREEN_SWEEP)
  {
    if(t.bSweeping)
    {  //exit sweep
      t.bSweeping = false;
      return;
    }
    /*
    if(t.iCursorPos == 0)
     GuiMenuRoot(true);
     else if(t.iCursorPos == 1)
     t.bSweeping = true;
     else
     ToggleCursor();
     */
    GuiScreenMenu(true);
  }
  ///////////////////////////////////
  // HARMONIC:
  else if(t.iGuiScreen == GUI_SCREEN_HARMONIC)
  {
    GuiScreenMenu(true);
  }
  ///////////////////////////////////
  // MEMORY:
  else if(t.iGuiScreen == GUI_SCREEN_MEMORY)
  {
    GuiScreenMenu(true);
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Toggle cursor between: position and adjustment states
void ToggleCursor(void)
{
  t.bCursorState = !t.bCursorState;
  if(t.bCursorState)
    BEEP_CURSOR_HI;
  else
    BEEP_CURSOR_LO;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
void IncCursor(bool bInc)
{
  if(bInc)
  {
    if(++t.iCursorPos > t.iCursorMax[t.iGuiScreen])
      t.iCursorPos = 0;
  }
  else
  {
    if(--t.iCursorPos < 0)
      t.iCursorPos = t.iCursorMax[t.iGuiScreen];
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GUI Change a channel frequency
void GuiChangeFreq(bool bInc)
{
  const int NUM_DIGITS = 8;
  if(t.iCursorPos == 0)
    return;

  int iChan = (t.iCursorPos-1) / NUM_DIGITS;
  if(iChan >= NUM_CHANNELS)
    iChan = NUM_CHANNELS - 1;

  int iDigit = NUM_DIGITS - ((t.iCursorPos-1) % NUM_DIGITS);

  unsigned long ulInc = 1;
  for (int i=1; i < iDigit; i++)
    ulInc *= 10;

  if(bInc)
  {
    v.lFreq[iChan]+=ulInc;
    if(v.lFreq[iChan] > MAX_FREQUENCY)
      v.lFreq[iChan] = MAX_FREQUENCY;
  }
  else
  {
    v.lFreq[iChan]-=ulInc;
    if(v.lFreq[iChan] < MIN_FREQUENCY)
      v.lFreq[iChan] = MIN_FREQUENCY;
  }
  DDS_SetFrequency(iChan, v.lFreq[iChan]);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GUI Change a sweep frequency
void GuiChangeSweep(bool bInc)
{
  /*
  if(t.iCursorPos <= 1)
   return;
   
   int iDigit = 8 - ((t.iCursorPos-2) % 8);
   int iChan = t.iMenuType - MENU_SWEEP_1;
   int iType = ((t.iCursorPos-2) / 8) % 3;
   
   unsigned long ulInc = 1;
   for (int i=1; i < iDigit; i++)
   ulInc *= 10;
   
   if(bInc)
   {
   v.lSweepFreq[iType][iChan]+=ulInc;
   if(v.lSweepFreq[iType][iChan] > MAX_FREQUENCY)
   v.lSweepFreq[iType][iChan] = MAX_FREQUENCY;
   }
   else
   {
   v.lSweepFreq[iType][iChan]-=ulInc;
   if(iType == 2)
   {
   if(v.lSweepFreq[iType][iChan] < 1)
   v.lSweepFreq[iType][iChan] = 1;
   }
   else
   {
   if(v.lSweepFreq[iType][iChan] < MIN_FREQUENCY)
   v.lSweepFreq[iType][iChan] = MIN_FREQUENCY;
   }
   }
   */
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   ____            _                   ___                   _       
//  / ___| _   _ ___| |_ ___ _ __ ___   |_ _|_ __  _ __  _   _| |_ ___ 
//  \___ \| | | / __| __/ _ \ '_ ` _ \   | || '_ \| '_ \| | | | __/ __|
//   ___) | |_| \__ \ ||  __/ | | | | |  | || | | | |_) | |_| | |_\__ \
//  |____/ \__, |___/\__\___|_| |_| |_| |___|_| |_| .__/ \__,_|\__|___/
//         |___/                                  |_|            
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Timer2 overflow interrupt vector handler, called every 4ms
//This is used to poll and filter the jogwheel button.
ISR(TIMER2_OVF_vect)
{
  HibernationCheck();

  const int MIN_TOUCH_WIDTH = 3;//4ms quanta
  const int MAX_TOUCH_WIDTH = 30;//4ms quanta
  const int MIN_INTER_TOUCH_PERIOD = 300;//1ms quanta

  static unsigned int uiFilter;
  static unsigned int uiCount = 0;
  static bool bTouchOld = false;

  bool bTouchNow = digitalRead(PIN_JOG_BUTTON);

  if(bTouchNow && !bTouchOld)
  {
    uiCount = 0;
  }
  else if(!bTouchNow && bTouchOld)
  {
    static unsigned long ulTouchOld = 0;
    unsigned long ulTouchNow = millis();
    if((ulTouchNow - ulTouchOld) >= MIN_INTER_TOUCH_PERIOD)
    {
      if(uiCount >= MIN_TOUCH_WIDTH && uiCount <= MAX_TOUCH_WIDTH)
      {  
        unsigned long ulPressNew = millis();
        //dont service pushes close to a jog action:
        if((ulPressNew-t.ulLastJog) >= MIN_INTER_TOUCH_PERIOD)
        {//valid press action:
          t.ulLastJog = ulPressNew;
          GuiScreenPressed();
        }
      }
    }
    ulTouchOld = ulTouchNow;
  }
  uiCount++;
  bTouchOld = bTouchNow;

  //also check joypad when sweeping
  if(t.bSweeping)
  {
    static int iKeyOld = 0;
    int iKeyNew = GetKey();
    if((iKeyOld == -1) && (iKeyNew != -1))
      t.bSweeping = false;
    iKeyOld = iKeyNew;        
  }

  return;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Interrupt from jogwheel A
ISR(INT1_vect)
{
  if(!digitalRead(PIN_JOG_GREY_A))
    return;

  JogWheelMoved(false);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Interrupt from jogwheel B
ISR(INT0_vect)
{
  if(!digitalRead(PIN_JOG_GREY_B))
    return;

  JogWheelMoved(true);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Do the correct GUI action for a jog wheel command
void JogWheelMoved(bool bClockWise)
{ 
  t.ulLastJog = millis();
  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  if(t.iGuiScreen == GUI_SCREEN_MENU)
  {
    IncCursor(!bClockWise);
  }
  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  else if(t.iGuiScreen == GUI_SCREEN_FREQUENCY)
  {
    if(t.bCursorState)
      GuiChangeFreq(bClockWise);
    else
      IncCursor(bClockWise);
  }
  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  else if(t.iGuiScreen == GUI_SCREEN_PHASE)
  {
  }
  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  else if(t.iGuiScreen == GUI_SCREEN_SWEEP)
  {
    if(t.bCursorState)
    {
      if(t.iCursorPos >= 2)
        GuiChangeSweep(bClockWise);
    }
    else
      IncCursor(bClockWise);
  }
  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  else if(t.iGuiScreen == GUI_SCREEN_HARMONIC)
  {
  }
  //////////////////////////////////////////////////////////////////////////////////////////////////////////
  else if(t.iGuiScreen == GUI_SCREEN_MEMORY)
  {
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Check user input on joypad and do corresponding action to GUI
void PollJoyPad(void)
{
  static bool bPushedLast;
  int nKey = GetKey();
  if(nKey == -1)//Nothing pressed
  {
    bPushedLast = false;
    return;
  }

  if(nKey == 1)//User pushing joypad button
  {
    if(!bPushedLast)
      GuiScreenPressed();
    bPushedLast = true;
    return;
  }

  //joypad must have been moved then:
  bPushedLast = false;

  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //MENU SCREEN:
  if(t.iGuiScreen == GUI_SCREEN_MENU)
  {
    switch(nKey)
    {
    case 0://LEFT
      break;
    case 2://DOWN
      IncCursor(false);
      BEEP_MOVE;
      break;
    case 3://UP
      IncCursor(true);
      BEEP_MOVE;
      break;
    case 4://RIGHT
      break;
    }
  }
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //FREQUENCY SCREEN:
  else if(t.iGuiScreen == GUI_SCREEN_FREQUENCY)
  {
    switch(nKey)
    {
    case 0://LEFT
      t.bCursorState = false;
      IncCursor(false);
      BEEP_MOVE;
      break;
    case 2://DOWN
      if(t.iCursorPos != 0)
      {
        t.bCursorState = true;
        GuiChangeFreq(false);
        BEEP_DEC;  
      }
      break;
    case 3://UP
      if(t.iCursorPos != 0)
      {
        t.bCursorState = true;
        GuiChangeFreq(true);
        BEEP_INC;
      }
      break;
    case 4://RIGHT
      t.bCursorState = false;
      IncCursor(true);
      BEEP_MOVE;
      break;
    }
  }
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //PHASE SCREEN:
  else if(t.iGuiScreen == GUI_SCREEN_PHASE)
  {
    switch(nKey)
    {
    case 0://LEFT
      break;
    case 2://DOWN
      break;
    case 3://UP
      break;
    case 4://RIGHT
      break;
    }
  }
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //SWEEP SCREEN:
  else if(t.iGuiScreen == GUI_SCREEN_SWEEP)
  {
    switch(nKey)
    {
    case 0://LEFT
      t.bCursorState = false;
      IncCursor(false);      
      BEEP_MOVE;
      break;
    case 2://DOWN
      if(t.iCursorPos >= 2)
      {
        t.bCursorState = true;
        GuiChangeSweep(false);
        BEEP_DEC;
      }
      break;
    case 3://UP
      if(t.iCursorPos >= 2)
      {
        t.bCursorState = true;
        GuiChangeSweep(true);
        BEEP_INC;  
      }
      break;
    case 4://RIGHT
      t.bCursorState = false;
      IncCursor(true);
      BEEP_MOVE;
      break;
    }
  }
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //HARMONIC SCREEN:
  else if(t.iGuiScreen == GUI_SCREEN_HARMONIC)
  {
    switch(nKey)
    {
    case 0://LEFT
      break;
    case 2://DOWN
      break;
    case 3://UP
      break;
    case 4://RIGHT
      break;
    }
  }
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //MEMORY SCREEN:
  else if(t.iGuiScreen == GUI_SCREEN_MEMORY)
  {
    switch(nKey)
    {
    case 0://LEFT
      break;
    case 2://DOWN
      break;
    case 3://UP
      break;
    case 4://RIGHT
      break;
    }
  }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Convert ADC value to key number
int GetKey(void)
{
  const int  adc_key_val[5] =
  { 
    70, 200, 450, 600, 900
  };

  const int NUM_KEYS = 5;
  int input = analogRead(PIN_ADC_JOYPAD);
  int k;
  for (k = 0; k < NUM_KEYS; k++)
    if (input < adc_key_val[k])
      return k;
  return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Check if the PSU is closing down and save state to EEPROM if it is
void HibernationCheck(void)
{
  static bool bLastGood = true;

  if(analogRead(PIN_ADC_PSU) < 1023)
  { //power is bad:
    //if(bLastGood == false)
    {
      //TODO:Save to eeprom
      Beep(4000,100);
      for(;;);
    }
    bLastGood = false;
  }
  else
    bLastGood = true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//   ____            _                    ___        _               _       
//  / ___| _   _ ___| |_ ___ _ __ ___    / _ \ _   _| |_ _ __  _   _| |_ ___ 
//  \___ \| | | / __| __/ _ \ '_ ` _ \  | | | | | | | __| '_ \| | | | __/ __|
//   ___) | |_| \__ \ ||  __/ | | | | | | |_| | |_| | |_| |_) | |_| | |_\__ \
//  |____/ \__, |___/\__\___|_| |_| |_|  \___/ \__,_|\__| .__/ \__,_|\__|___/
//         |___/                                        |_|                  
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Beep noise, freq in Hz, length in milliseconds
void Beep(int Freq, int Length)
{
  if(!v.bPiezoEnabled)
    return;

  SPCR = 0x00;   // disable SPI master
  pinMode(PIN_BUZZER, OUTPUT);

  const int MAX_HZ = 8000;
  if(Freq > MAX_HZ)
    Freq = MAX_HZ;

  int period = (int)(500000/(long)Freq);
  int loops = ((long)Length * (long)500) / period;
  for(int i = 0; i < loops; i++)
  {
    digitalWrite(PIN_BUZZER, true);
    delayMicroseconds(period);
    digitalWrite(PIN_BUZZER, false);
    delayMicroseconds(period);
  }

  SPCR = 0x51;   // enable SPI master, fosc/16 = 1MH
}
/*
////////////////////////////////////////////////////////////////////////////////////////////////////////////
 // Start 3D frequency sweeping
 void Perform3DSweep(void)
 {
 BEEP_MENU;
 const int X1=4, Y1=119, X2=127, Y2=127;
 LCD.Box(X1-1, Y1-1, X2+1, Y2+1, COLOR_TEXT);
 LCD.Box(X1, Y1, X2, Y2, COLOR_GFX1);
 int iProgress = X1;
 
 const int CH1=0,CH2=1,CH3=2;
 unsigned long ulSteps = ((v.lSweepFreq[1][CH1]-v.lSweepFreq[0][CH1])/v.lSweepFreq[2][CH1]) + 1;
 ulSteps *= ((v.lSweepFreq[1][CH2]-v.lSweepFreq[0][CH2])/v.lSweepFreq[2][CH2]) + 1;
 ulSteps *= ((v.lSweepFreq[1][CH3]-v.lSweepFreq[0][CH3])/v.lSweepFreq[2][CH3]) + 1;
 unsigned long ulDivide = ulSteps / (X2-X1);
 unsigned long ulCount = 0;
 
 int iInput = 0;
 int iInputHi = -1;
 for(long lF3 = v.lSweepFreq[0][CH3]; lF3 <= v.lSweepFreq[1][CH3]; lF3 += v.lSweepFreq[2][CH3])
 {
 DDS_SetFrequency(CH3, lF3);
 for(long lF2 = v.lSweepFreq[0][CH2]; lF2 <= v.lSweepFreq[1][CH2]; lF2 += v.lSweepFreq[2][CH2])
 {
 DDS_SetFrequency(CH2, lF2);
 for(long lF1 = v.lSweepFreq[0][CH1]; lF1 <= v.lSweepFreq[1][CH1]; lF1 += v.lSweepFreq[2][CH1])
 {
 DDS_SetFrequency(CH1, lF1);
 iInput = analogRead(PIN_ADC_INPUT);
 if(iInput > iInputHi)
 {  //store highest ADC input position
 iInputHi = iInput;
 v.lFreq[CH1] = lF1*10;  
 v.lFreq[CH2] = lF2*10;  
 v.lFreq[CH3] = lF3*10;  
 }
 if(++ulCount >= ulDivide)
 {
 ulCount = 0;
 LCD.Line(iProgress,Y1,iProgress,Y2,COLOR_GFX2);
 iProgress++;
 }  
 if(!t.bSweeping)
 {
 GuiMenuRoot(true);
 return;
 }  
 }
 }
 }
 t.bSweeping = false;
 GuiMenuRoot(true);
 }
 */
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Start 1D spectrum
void PerformSweep(void)
{
  BEEP_MENU;
  LCD.SmallString(0, 40, "...SWEEPING...", 100);

  while(t.bSweeping)
  {
    for(long f = v.lSweepFreq[0][0]; f < v.lSweepFreq[1][0] && t.bSweeping; f++)
      DDS_SetFrequency(0,f);
  }
  GuiScreenMenu(true);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  specify zero based channel and Frequency in Hz
int iClock;
void DDS_SetFrequency(int iChan, unsigned long ulFreq)
{
  unsigned long tuning_word = (ulFreq * pow(2, 32)) / DDS_CLOCK;
  iClock = PIN_DDS_CLK1 + iChan;

  DISABLE_JOG_INTERRUPT;
  ////////////////////////////////
  // CRITICAL SECTION START
  ////////////////////////////////
  digitalWrite (PIN_DDS_LOAD, LOW);
  for(int i = 0; i < 32; i++)
  {
    DDS_BitOut(tuning_word & 1);
    tuning_word >>= 1;
  }
  DDS_ByteOut(0x09);
  digitalWrite (PIN_DDS_LOAD, HIGH);
  ////////////////////////////////
  // CRITICAL SECTION END
  ////////////////////////////////
  ENABLE_JOG_INTERRUPT;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output byte to serial data line of releavant DDS module
void DDS_ByteOut(unsigned char byte)
{
  for (int i = 0; i < 8; i++)
  {
    DDS_BitOut(byte & 1);
    byte >>= 1;
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Write one bit to serial data line of releavant DDS module
void DDS_BitOut(bool bHi)
{
  digitalWrite(iClock, LOW);
  digitalWrite(PIN_DDS_DATA, bHi);
  digitalWrite(iClock, HIGH);
}







